import couchdb
import json
import os
import xmltodict

from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import render, redirect

from .models import GPXFile
from .forms import GPXForm

def index(request):
    return render(request, "main/index.htm", {})


def about(request):
    return render(request, "main/about.htm", {})

    
def job(request):
	documents = GPXFile.objects.all()
	return render(request, "main/job.htm", { 'gpx': documents })

    
def gpxUpload(request):
    if request.method == 'POST':
        form = GPXForm(request.POST, request.FILES)
        if form.is_valid():
            #form.save()
            handleUploadedFile({"document": request.FILES['document'],
                                "userId": request.POST["userId"],
                                "trackId": request.POST["trackId"],
                                "description": request.POST["description"]})
            return redirect('job')
    else:
        form = GPXForm()
    return render(request, 'main/gpx_upload.htm', {
        'form': form
    })
    
    
def handleUploadedFile(data):
    contentsGPX = data['document']
    userId = data['userId']
    trackId = data['trackId']
    description = data['description']
    
    fileOutName = os.path.join(settings.MEDIA_ROOT, 'gpx', 'temp', 'document.dat')
        
#    documentDict = {"userId": userId,
#                    "trackId": trackId,
#                    "description": description}
    documentDict = transform(contentsGPX, userId, trackId)
    documentDict['description'] = description
    
    jsonS = json.dumps(documentDict) 
        
    with open(fileOutName, 'w') as fileToSaveData:
        fileToSaveData.write(jsonS)
        
    couch = couchdb.Server('http://userL:userP@127.0.0.1:5984/')
    db = couch['project']
    db.save(documentDict)
   

def transform(contents, userId, trackId):
    xml = xmltodict.parse(contents)
        
    trk = xml['gpx']['trk'] # xml is of OrderedDict type
    trkseg = trk['trkseg']
    points = processSegment(trkseg)
        
    points[0]['extensions']['keys'].append('type')
    points[0]['extensions']['type'] = "trackFirstPoint"
            
    points[-1]['extensions']['keys'].append('type')
    points[-1]['extensions']['type'] = "trackLastPoint"
            
    document = {"userId": userId,
                "trackId": trackId,
                "points": points}
		    
    return document
		
		
def processSegment(trkseg):
    points = []
    
    for pt in trkseg['trkpt']:
        extensions = pt["extensions"]["ns3:TrackPointExtension"]
        point = {"lat": pt['@lat'],
                 "lon": pt['@lon'],
                 "ele": pt['ele'],
                 "time": pt['time'],
                 "extensions": {"keys": ["temp"],
                                "temp": [extensions["ns3:atemp"]]}}
                                
        points.append(point)
     
    return points
    
    
def trackLength(request):
    couch = couchdb.Server('http://userL:userP@127.0.0.1:5984/')
    db = couch['project']
    
    tracks = []
    
    for item in db.view('queries/getTrackLength'):
        tracks.append(item.value)
    
	
    return render(request, "main/track_length.htm", {"tracks": tracks})    


def allTracksLength(request):
    couch = couchdb.Server('http://userL:userP@127.0.0.1:5984/')
    db = couch['project']
    
    users = []
    dictionary = {}
    
    for item in db.view('queries/getAllTracksLength'):
        dictionary = item.value
        # In this case, as it is map-reduce view, there should be only one item
        # Just in case I break the loop
        break
        
    for key in dictionary:
        users.append({"userId": key,
		              "allTracksLength": dictionary[key]})    
        
    return render(request, "main/tracks_length_all.htm", {"users": users})
    
